/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.definition.activity;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.logging.Logger;

import org.ow2.orchestra.definition.element.CompensationHandler;
import org.ow2.orchestra.runtime.BpelExecution;
import org.ow2.orchestra.runtime.ScopeRuntime;

/**
 * the runtime behaviour of the compensateScope activity.
 *
 * @author Tom Baeyens, Charles Souillard
 */
public class CompensateScope extends AbstractActivity  {

    /** serialVersionUID. */
    private static final long serialVersionUID = 1L;

    /** the target scope attribute */
    private String target;

    private static Logger log = Logger.getLogger(CompensateScope.class.getName());

    protected CompensateScope() {
        super();
    }

    public CompensateScope(String uuid) {
        super(uuid);
    }

    /**
     * business logic.
     *
     * @param execution
     *            execution
     */
    public void executeActivity(BpelExecution execution) {
        List<CompensationHandler> chs = new ArrayList<CompensationHandler>();
        
        BpelExecution parent = (BpelExecution) execution;;
        while(parent != null) {
            for (ScopeRuntime scope : parent.getScopeRuntimes()) {
                for (CompensationHandler compensationHandler : scope.getCompensationHandlers()) {
                    chs.add(compensationHandler);
                }
            }
            
            if(parent != execution.getParent()) {
                parent = (BpelExecution) execution.getParent();
            } else {
                parent = null;
            }
        }
        
        Iterator<CompensationHandler> chsIt = chs.iterator();
        if (target == null) {
            // Execution of all CompensationHandler in the list
            while (chsIt.hasNext()) {
                CompensationHandler ch = chsIt.next();
                if(ch != null) {
                    ch.getActivity().setParentNode(execution.getNode());
                    execution.getNode().getNodes().add(ch.getActivity());
                    execution.execute(ch.getActivity());
                }
            }
        } else {
            // Execution of a specific CompensationHandler in the list
            while (chsIt.hasNext()) {
                CompensationHandler ch = chsIt.next();
                if(ch != null) {
                    if (ch.getName().equals(target)) {
                        // specific compensationHandler matched
                        ch.getActivity().setParentNode(execution.getNode());
                        execution.getNode().getNodes().add(ch.getActivity());
                        execution.execute(ch.getActivity());
                        break;
                    }
                }
            }
        }

    }

    /**
     * Signal method of the NodeBehaviour.
     *
     * @param execution execution
     * @param signal signal
     */
    @Override
    public void signal(BpelExecution execution, String signal, Map<String, Object> parameters) {
        executeSources(execution);
    }

    public String getTarget() {
        return target;
    }

    public void setTarget(String target) {
        this.target = target;
    }

}
