/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.definition.activity;

import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.ow2.orchestra.definition.element.OnAlarm;
import org.ow2.orchestra.lang.Expression;
import org.ow2.orchestra.runtime.BpelExecution;
import org.ow2.orchestra.services.MutexRepository;
import org.ow2.orchestra.services.MutexRepository.InstanceLock;

/**
 * the runtime behaviour of the wait activity.
 * 
 * @author Tom Baeyens, Charles Souillard, Pascal Verdage
 */
public class Wait extends AbstractActivity {

  /** serialVersionUID. */
  private static final long serialVersionUID = 1L;

  private static Logger log = Logger.getLogger(Wait.class.getName());
  
  private Expression durationExpression = null;

  private Expression deadlineExpression = null;

  protected Wait() {
    super();
  }

  public Wait(String uuid) {
    super(uuid);
  }

  /**
   * business logic.
   * 
   * @param execution
   *            execution
   */
  /*
   * sample duration: 'P1Y2M15DT2H59M59S':
   *    1 year, 2 months, 15 days, 2 hours, 59 minutes, 59 seconds
   * sample deadline: '2008-03-15T18:00'
   */
  public void executeActivity(BpelExecution execution) {
    OnAlarm.createTimer(execution, 
        deadlineExpression, durationExpression, "timeout");
    execution.waitForSignal();
  }

  public void setDurationExpression(Expression durationExpression) {
    this.durationExpression = durationExpression;
  }

  public void setDeadlineExpression(Expression deadlineExpression) {
    this.deadlineExpression = deadlineExpression;
  }

  @Override
  public void signal(BpelExecution execution, String signal,
      Map<String, Object> parameters) {
    BpelExecution bpelInstance = (BpelExecution) execution.getProcessInstance();
    InstanceLock lock = MutexRepository.lockInstance(bpelInstance);
    try {
      if (log.isLoggable(Level.FINE)) {
        log.fine("timer reached execution time at " + System.currentTimeMillis());
      }
      executeSources(execution);
    } finally {
      MutexRepository.unlockInstance(lock);
    }
  }

}
