package org.ow2.orchestra.definition.element;

import java.util.Date;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.xml.datatype.Duration;

import org.jbpm.pvm.env.Environment;
import org.jbpm.pvm.internal.job.TimerImpl;
import org.jbpm.pvm.internal.model.NodeImpl;
import org.jbpm.pvm.job.Timer;
import org.jbpm.pvm.session.TimerSession;
import org.ow2.novabpm.util.Misc;
import org.ow2.orchestra.exception.OrchestraException;
import org.ow2.orchestra.lang.Expression;
import org.ow2.orchestra.runtime.BpelExecution;


public class OnAlarm {

  private static Logger log = Logger.getLogger(OnAlarm.class.getName());
 
  private String uuid = Misc.getUniqueId("onAlarm");
  
  private Expression durationExpression;

  private Expression deadlineExpression;
  
  protected NodeImpl activity;

  public void setDurationExpression(Expression durationExpression) {
    this.durationExpression = durationExpression;
  }
  public void setDeadlineExpression(Expression deadlineExpression) {
    this.deadlineExpression = deadlineExpression;
  }
  public NodeImpl getActivity() {
    return activity;
  }
  public void setActivity(NodeImpl activity) {
    this.activity = activity;
  }
  public Expression getDurationExpression() {
    return durationExpression;
  }
  public Expression getDeadlineExpression() {
    return deadlineExpression;
  }
  
 
  
  public static void createTimer(BpelExecution execution, Expression deadlineExpression, 
      Expression durationExpression, String signalName) {
    Date date = null;

    if (deadlineExpression != null) {
      date = deadlineExpression.getEvaluator().evaluateDeadline(execution).toGregorianCalendar().getTime();
      if (log.isLoggable(Level.FINE)) {
        log.fine("waiting until " + date);
      }

    } else if (durationExpression != null) {
      Duration duration = durationExpression.getEvaluator().evaluateDuration(execution);
      date = new Date();
      duration.addTo(date);
      if (log.isLoggable(Level.FINE)) {
        log.fine("waiting for " + duration);
      }

    } else {
      throw new OrchestraException("invalid timer definition");
    }
    
    execution.createTimer(null, signalName, null, date, null, null, null);
  }
  
  public String getUuid() {
    return uuid;
  }
}
