package org.ow2.orchestra.env;

import java.io.InputStream;
import java.net.URL;

import javax.xml.parsers.DocumentBuilderFactory;

import org.jbpm.pvm.env.EnvironmentFactory;
import org.jbpm.pvm.internal.env.PvmEnvironmentFactoryParser;
import org.jbpm.pvm.internal.xml.Bindings;
import org.jbpm.pvm.internal.xml.Parse;
import org.ow2.orchestra.env.binding.HibernateBpelPersistenceServiceBinding;
import org.ow2.orchestra.env.binding.ImplementationBinding;
import org.xml.sax.InputSource;

public class BpelEnvironmentParser extends PvmEnvironmentFactoryParser {

  private static PvmEnvironmentFactoryParser instance;

  public static final String CLASSPATH_URL = "classpath://";

  public static synchronized PvmEnvironmentFactoryParser getInstance() {
    if (instance == null) {
      instance = new BpelEnvironmentParser();
      // Get bindings
      Bindings bindings = instance.getEnvironmentXmlParser().getBindings();
      // Use the same bindings in block and application contexts
      instance.getEnvironmentFactoryXmlParser().setBindings(bindings);
      // Add orchestra bindings
      bindings.addBinding(new HibernateBpelPersistenceServiceBinding());
      bindings.addBinding(new ImplementationBinding("repository"));
      bindings.addBinding(new ImplementationBinding("invoker"));
      bindings.addBinding(new ImplementationBinding("publisher"));
    }
    return instance;
  }

  public static EnvironmentFactory parseEnvironmentFactoryFromXmlString(
      String xmlString) {
    Parse parse = getInstance().createParse();
    EnvironmentFactory factory = (EnvironmentFactory) parse.setString(xmlString).execute().getDocumentObject();
    parse.checkProblems("environment");
    return factory;
  }

  public static synchronized EnvironmentFactory parseEnvironmentFactoryFromResource(
      String resource) {
    Parse parse = getInstance().createParse();
    EnvironmentFactory factory = (EnvironmentFactory) parse.setResource(resource).execute().getDocumentObject();
    parse.checkProblems("environment");
    return factory;
  }

  public synchronized DocumentBuilderFactory getDocumentBuilderFactory() {
    documentBuilderFactory = newDocumentBuilderFactory();
    documentBuilderFactory.setNamespaceAware(true);
    documentBuilderFactory.setValidating(true);
    // ignore white space can only be set if parser is validating
    documentBuilderFactory.setIgnoringElementContentWhitespace(true);
    // select xml schema as the schema language (a.o.t. DTD)
    documentBuilderFactory.setAttribute(
        "http://java.sun.com/xml/jaxp/properties/schemaLanguage",
        "http://www.w3.org/2001/XMLSchema");
    //  set schema sources
    URL url = Thread.currentThread().getContextClassLoader().getResource("environment-orchestra.xsd");
    documentBuilderFactory.setAttribute(
        "http://java.sun.com/xml/jaxp/properties/schemaSource", url.toExternalForm());
    return documentBuilderFactory;
  }
  
  /* (non-Javadoc)
   * @see org.jbpm.xml.Parser#resolveEntity(java.lang.String, java.lang.String)
   */
  @Override
  public InputSource resolveEntity(String publicId, String systemId) {
    if (systemId.startsWith(CLASSPATH_URL)) {
      String localpart = systemId.substring(CLASSPATH_URL.length());
      InputStream inStream = Thread.currentThread().getContextClassLoader().getResourceAsStream(localpart);
      if (inStream != null) {
        return new InputSource(inStream);
      }
    }
    return super.resolveEntity(publicId, systemId);
  }
}
